/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include "globals.h"
#include "BestFit.h"

bool BestFit::find_hole(const string id, const int size)
{
    int min_extra = MAX_BLOCKS;
    auto min_segment_list_iterator = segment_list.end();

    // Start searching from the start of the segment list
    // for the best possible fit.
    for (segment_list_iterator = segment_list.begin();
         segment_list_iterator != segment_list.end();
         segment_list_iterator++)
    {
        Segment *segment = *segment_list_iterator;

        // Best fit search.
        if (segment->size < 0)
        {
            int extra = -segment->size - size;

            if (extra == 0) return true;  // found an exact match

            if ((extra > 0) && (extra < min_extra))
            {
                min_extra = extra;
                min_segment_list_iterator = segment_list_iterator;
            }
        }
    }

    segment_list_iterator = min_segment_list_iterator;

    return min_extra != MAX_BLOCKS;
}
