/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include <string>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include "globals.h"
#include "Request.h"
#include "Segment.h"

using namespace std;

const int Request::SIZES[]      = { 3, 5, 7, 11, 13, 17, 19, 23, 29 };
const int Request::SIZES_LENGTH = sizeof(SIZES)/sizeof(int);

Segment *Request::request_allocation()
{
    sprintf(message.id, "%c%05d", name, ++counter);
    message.size = SIZES[rand()%SIZES_LENGTH];
    write(fd, (char *)&message, sizeof(message));

    string id = message.id;
    Segment *alloc = new Segment(id, message.size);
    allocation_vector.push_back(alloc);

    cout << "Requester " << name << ": id = " << message.id
         << ", size = " << message.size << " ... ";
    cout.flush();

    return alloc;
}

Segment *Request::request_deallocation()
{
    if (allocation_vector.size() == 0) return nullptr;

    int i = rand()%allocation_vector.size();
    Segment *alloc = allocation_vector[i];

    sprintf(message.id, "%s", alloc->id.c_str());
    message.size = -1;
    write(fd, (char *)&message, sizeof(message));

    allocation_vector.erase(allocation_vector.begin() + i);

    cout << "Requester " << name << ": id = " << alloc->id << endl;
    return alloc;
}

bool Request::next_request()
{
    bool read_succeeded = false;

    if (buffer_index == 0)
    {
        do
        {
            read_size = read(fd, buffer, BUFFER_SIZE);

            switch (read_size)
            {
                case -1:
                {
                    if (errno == EAGAIN)
                    {
                        // Pipe empty. Snooze and try again.
//                        printf("(empty)\n");
                        usleep(U_SECONDS);
                        break;
                    }
                    else exit(-11);
                }

                case 0:
                {
                    // End of file.
                    return false;
                }

                default:
                {
                    read_succeeded = true;

//                    printf("(read %d)\n", (int)read_size);
                    break;
                }
            }
        } while (!read_succeeded);
    }

    char *to   = (char *)&message;
    char *from = &buffer[buffer_index];
    for (int i = 0; i < sizeof(message); i++) *to++ = *from++;

    buffer_index += sizeof(message);
    if (buffer_index >= read_size) buffer_index = 0;

    return true;
}
