/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#ifndef REQUEST_H_
#define REQUEST_H_

#include <string>
#include <vector>
#include <map>

#include "Segment.h"

using namespace std;

class Request
{
public:
    /**
     * Constructor for the memory manager.
     */
    Request(int fd)
        : name('\0'), fd(fd), counter(0), read_size(0), buffer_index(0) {}

    /**
     * Constructor for a requester.
     */
    Request(char name, int fd)
        : name(name), fd(fd), counter(0), read_size(0), buffer_index(0) {}

    virtual ~Request() {}

    string get_id() const { return string(message.id); }
    int get_size()  const { return message.size; }

    Segment *request_allocation();
    Segment *request_deallocation();
    bool next_request();

private:
    char name;
    int fd;

    int counter;
    ssize_t read_size;

    static const int SIZES[];
    static const int SIZES_LENGTH;

    // Allocations for this requester
    vector<Segment *> allocation_vector;

    static const int ID_SIZE = 10;
    struct
    {
        char id[ID_SIZE];
        int size;
    } message;

    static const int BUFFER_SIZE = 80;
    char buffer[BUFFER_SIZE];
    int buffer_index;
};

#endif /* REQUEST_H_ */
